@@ -33,52 +33,106 @@
 #define _XGI_H_
 
 /***************
-
-#define DEBUG2
 #define DEBUG 
 #define DEBUG1
+#define DEBUG2
 #define DEBUG3
 #define DEBUG4
 #define DEBUG5
+#define DEBUGI2C
 *****************/
 
+#ifndef XGI_VIDEO_HW /* avoid compile error in xgi_videohw.c; weird!  */
+/* Jong 07/27/2009; use run-time debug instead except for HW acceleration routines */
+extern Bool g_bRunTimeDebug;
+#define RUNTIMEDEBUG(p)		if(g_bRunTimeDebug)p;
+
+/* Jong@08052009 */
+#ifdef  DEBUGI2C
+#define PDEBUGI2C(p)       p
+#else
+#define PDEBUGI2C(p)
+#endif
 
 #ifdef  DEBUG
-#define PDEBUG(p)       p
+#define PDEBUG(p)       RUNTIMEDEBUG(p)
 #else
 #define PDEBUG(p)
 #endif
 
 #ifdef DEBUG1
-#define PDEBUG1(p) p
+#define PDEBUG1(p)		RUNTIMEDEBUG(p)
 #else
 #define PDEBUG1(p)
 #endif
 
 #ifdef DEBUG2
-#define PDEBUG2(p) p
+#define PDEBUG2(p)		RUNTIMEDEBUG(p)
 #else
 #define PDEBUG2(p)
 #endif
 
 #ifdef DEBUG3
-#define PDEBUG3(p) p
+#define PDEBUG3(p)		RUNTIMEDEBUG(p)
 #else
 #define PDEBUG3(p)
 #endif
 
 #ifdef DEBUG4
-#define PDEBUG4(p) p
+#define PDEBUG4(p)		RUNTIMEDEBUG(p)
 #else
 #define PDEBUG4(p)
 #endif
 
 #ifdef  DEBUG5
-#define PDEBUG5(p)       p
+#define PDEBUG5(p)		RUNTIMEDEBUG(p)
 #else
 #define PDEBUG5(p)
 #endif
 
+#ifdef  CDEBUG
+#define CPDEBUG(p)       p
+#else
+#define CPDEBUG(p)
+#endif
+
+#ifdef CDEBUG1
+#define CPDEBUG1(p) p
+#else
+#define CPDEBUG1(p)
+#endif
+
+#ifdef CDEBUG2
+#define CPDEBUG2(p) p
+#else
+#define CPDEBUG2(p)
+#endif
+
+#ifdef CDEBUG3
+#define CPDEBUG3(p) p
+#else
+#define CPDEBUG3(p)
+#endif
+
+#ifdef CDEBUG4
+#define CPDEBUG4(p) p
+#else
+#define CPDEBUG4(p)
+#endif
+
+#ifdef  CDEBUG5
+#define CPDEBUG5(p)       p
+#else
+#define CPDEBUG5(p)
+#endif
+
+#ifdef  ACCELDEBUG
+#define PACCELDEBUG(p)       p
+#else
+#define PACCELDEBUG(p)
+#endif
+#endif
+
 /* Always unlock the registers (should be set!) */
 #define UNLOCK_ALWAYS
 
@@ -92,29 +146,74 @@
 #include "xf86Cursor.h"
 #include "xf86xv.h"
 #include "compiler.h"
-#include "xaa.h"
-#include "vgaHW.h"
-#include "vbe.h"
 
 #ifdef XORG_VERSION_CURRENT
 #include "xorgVersion.h"
+
+/* #if XORG_VERSION_CURRENT > XORG_VERSION_NUMERIC(7,0,0,0,0) */
+#if ((XORG_VERSION_CURRENT > XORG_VERSION_NUMERIC(7,0,0,0,0)) || (XORG_VERSION_CURRENT < XORG_VERSION_NUMERIC(2,0,0,0,0)) )
+#define XGIISXORGPOST70
 #endif
 
+#if (XORG_VERSION_CURRENT >= XORG_VERSION_NUMERIC(6,9,0,0,0) )
+#define XGI_USE_EXA
+#endif
+
+#endif
+
+/* Jong 01/13/2009; support EXA */
+#ifdef HAVE_XAA_H
+#define XGI_USE_XAA 
+#endif
+/* #define XGI_USE_EXA */
+
+#ifdef XGI_USE_XAA
+#include "xaa.h"
+#endif
+#ifdef XGI_USE_EXA
+#include "exa.h"
+#endif
+
+#include "vgaHW.h"
+#include "vbe.h"
+
+/*
+#ifdef XORG_VERSION_CURRENT
+#include "xorgVersion.h"
+#endif */
+
+/* #define XGIISXORGPOST70 */
+
 #include "xgi_pci.h"
 #include "osdef.h"
 #include "vgatypes.h"
 #include "vb_struct.h"
 
+#include "compat-api.h"
+
 #ifdef XF86DRI
 #define XGINEWDRI
 #undef XGINEWDRI2
+
+#ifdef XF86_VERSION_CURRENT
 #if XF86_VERSION_CURRENT >= XF86_VERSION_NUMERIC(4,4,99,99,0)	/* Adapt this when the time has come */
 #define XGINEWDRI2
 #endif
+#endif /* XF86_VERSION_CURRENT */
+
 #include "xf86drm.h"
 #include "sarea.h"
 #define _XF86DRI_SERVER_
+
+/* Jong@09032009 */
+#ifdef XORG_VERSION_CURRENT
+#if ( (XORG_VERSION_CURRENT >= XORG_VERSION_NUMERIC(6,9,0,0,0)) || (XORG_VERSION_CURRENT <= XORG_VERSION_NUMERIC(2,0,0,0,0)) )
+#include "X11/dri/xf86dri.h"
+#else
 #include "xf86dri.h"
+#endif
+#endif
+
 #include "dri.h"
 #include "GL/glxint.h"
 #include "xgi_dri.h"
@@ -152,7 +251,8 @@
 #define XGIGAMMA		/* Include code for gamma correction */
 #endif
 
-#if 1				/* Include code for color hardware cursors */
+/* Jong 09/28/2007; disable this because it causes cursor drawing incorrectly */
+#if 0				/* Include code for color hardware cursors */
 #define XGI_ARGB_CURSOR
 #endif
 
@@ -162,7 +262,6 @@
 
 #ifdef XGIMERGED
 #ifdef XGIXINERAMA
-#define NEED_REPLIES  		/* ? */
 #define EXTENSION_PROC_ARGS void *
 #include "extnsionst.h"  	/* required */
 #include <X11/extensions/panoramiXproto.h>  	/* required */
@@ -176,7 +275,7 @@
 #undef XGI315DRI		/* define this if dri is adapted for 315/330 series */
 
 #ifndef PCI_VENDOR_XGI
-#define PCI_VENDOR_XGI 		        0x18CA
+#define PCI_VENDOR_XGI 		    0x18CA
 #endif
 #ifndef PCI_CHIP_XGIXG40
 #define PCI_CHIP_XGIXG40 		0x0040
@@ -185,6 +284,16 @@
 #define PCI_CHIP_XGIXG20 		0x0020
 #endif
 
+/* Jong 09/18/2007; patch to GIT */
+#ifndef PCI_CHIP_XGIXG27
+#define PCI_CHIP_XGIXG27 		0x0027
+#endif
+
+/* Jong 01/07/2008; support New XG21 */
+#ifndef PCI_CHIP_XGIXG21
+#define PCI_CHIP_XGIXG21 		0x0021
+#endif
+
 #define CONFIG_DRM_XGI
 
 #define XGI_NAME                "XGI"
@@ -509,9 +618,9 @@ typedef struct {
     unsigned long       FbAddress;      /* VRAM physical address (in DHM: for each Fb!) */
     unsigned long       realFbAddress;  /* For DHM/PCI mem mapping: store global FBAddress */
     unsigned char *     FbBase;         /* VRAM virtual linear address */
-    CARD32              IOAddress;      /* MMIO physical address */
+    uint32_t              IOAddress;    /* MMIO physical address */
     unsigned char *     IOBase;         /* MMIO linear address */
-    IOADDRESS           IODBase;        /* Base of PIO memory area */
+    unsigned long           IODBase;        /* Base of PIO memory area */
 #ifdef __alpha__
     unsigned char *     IOBaseDense;    /* MMIO for Alpha platform */
 #endif
@@ -531,6 +640,7 @@ typedef struct {
     int                 numDGAModes;
     Bool                DGAactive;
     Bool                NoAccel;
+    Bool				useEXA;  /* Jong 01/13/2009; support EXA */
     Bool                NoXvideo;
     Bool                TurboQueue;
     int                 ForceCRT1Type;
@@ -549,7 +659,7 @@ typedef struct {
     int                 CommandReg;
 
     Bool                HWCursor;
-    CARD16		CursorSize;  		/* Size of HWCursor area (bytes) */
+    uint16_t		CursorSize;  		/* Size of HWCursor area (bytes) */
     xf86CursorInfoPtr   CursorInfoPtr;
     unsigned            CursorOffset;
 
@@ -560,7 +670,22 @@ typedef struct {
 
     XGIRegRec           SavedReg;
     XGIRegRec           ModeReg;
+
+#ifdef XGI_USE_XAA
     XAAInfoRecPtr       AccelInfoPtr;
+#endif
+#ifdef XGI_USE_EXA /* Jong 01/13/2009; support EXA */
+    ExaDriverPtr		EXADriverPtr;
+    int			fillPitch, fillBpp;
+    uint32_t		fillDstBase;
+    int			copyBpp;
+    int			copySPitch, copyDPitch;
+    uint32_t		copySrcBase, copyDstBase;
+    int			copyXdir, copyYdir;
+    ExaOffscreenArea*	exa_scratch;
+    unsigned int 		exa_scratch_next;
+#endif
+
     CloseScreenProcPtr  CloseScreen;
     Bool        	(*ModeInit)(ScrnInfoPtr pScrn, DisplayModePtr mode);
     void        	(*XGISave)(ScrnInfoPtr pScrn, XGIRegPtr xgireg);
@@ -627,6 +752,8 @@ typedef struct {
 
     int 		Rotate;
 
+    BOOLEAN		HaveCustomModes; /* Jong 07/27/2009; support customer modes */
+
     /* ShadowFB support */
     Bool 		ShadowFB;
     unsigned char 	*ShadowPtr;
@@ -666,19 +793,19 @@ typedef struct {
     unsigned char 	LCDon;
     Bool 		Blank;
     int 		CRT1off;		/* 1=CRT1 off, 0=CRT1 on */
-    CARD16 		LCDheight;		/* Vertical resolution of LCD panel */
-    CARD16 		LCDwidth;		/* Horizontal resolution of LCD panel */
+    uint16_t 		LCDheight;		/* Vertical resolution of LCD panel */
+    uint16_t 		LCDwidth;		/* Horizontal resolution of LCD panel */
     vbeInfoPtr 		pVbe;			/* For VESA mode switching */
     UCHAR ScratchSet[16];
     MonitorRangeRec CRT1Range,CRT2Range;
 
 #ifdef XGIDUALHEAD
-    BOOL 		DualHeadMode;		/* TRUE if we use dual head mode */
-    BOOL 		SecondHead;		/* TRUE is this is the second head */
+    Bool 		DualHeadMode;		/* TRUE if we use dual head mode */
+    Bool 		SecondHead;		/* TRUE is this is the second head */
     XGIEntPtr 		entityPrivate;		/* Ptr to private entity (see above) */
 #endif
     XGIFBLayout         CurrentLayout;		/* Current framebuffer layout */
-    BOOL		Primary;		/* Display adapter is primary */
+    Bool		Primary;		/* Display adapter is primary */
     xf86Int10InfoPtr    pInt;			/* Our int10 */
     
     /**
@@ -704,6 +831,8 @@ typedef struct {
 #endif
     unsigned long       ChipFlags;
     unsigned long       XGI_SD_Flags;
+    BOOLEAN		UseHWARGBCursor;
+    BOOLEAN		HWARGBCursor;
     int                 vb;
     BOOLEAN		restorebyset;
     BOOLEAN		nocrt2ddcdetection;
@@ -713,17 +842,23 @@ typedef struct {
     BOOLEAN		XvDefDisableGfx, XvDefDisableGfxLR;
     BOOLEAN		XvUseMemcpy;
     int			XvGammaRed, XvGammaGreen, XvGammaBlue;
-    CARD8		XvGammaRampRed[256], XvGammaRampGreen[256], XvGammaRampBlue[256];
+    uint8_t		XvGammaRampRed[256], XvGammaRampGreen[256], XvGammaRampBlue[256];
     BOOLEAN		disablecolorkeycurrent;
-    CARD32		colorKey;
-    CARD32		MiscFlags;
+    uint32_t		colorKey;
+    uint32_t		MiscFlags;
+
+#ifdef XGI_USE_XAA
     FBLinearPtr		AccelLinearScratch;
+#endif
+
     float		zClearVal;
     unsigned long	bClrColor, dwColor;
     int			AllowHotkey;
     BOOLEAN		enablexgictrl;
     short		Video_MaxWidth, Video_MaxHeight;
     short               scrnPitch2;
+    int	                CurXPreset ;
+    int                 CurYPreset ;
     unsigned long	mmioSize;
 #ifdef XGIMERGED
     Bool		MergedFB, MergedFBAuto;
@@ -767,6 +902,14 @@ typedef struct {
     Bool		v4l_videoin;
     int			v4l_devnum;	/* v4l device number, 0,1,2....*/
 //~::::    
+
+	int			TargetRefreshRate; 
+	Bool		IgnoreDDC; 
+
+	Bool		Non_DDC_DefaultMode;
+	int			Non_DDC_DefaultResolutionX ;
+	int			Non_DDC_DefaultResolutionY ;
+	int			Non_DDC_DefaultRefreshRate ;
 } XGIRec, *XGIPtr;
 
 #ifdef XGIDUALHEAD
@@ -835,6 +978,7 @@ int compute_vclk(int Clock, int *out_n, int *out_dn, int *out_div,
 void XGI_WaitBeginRetrace(XGIIOADDRESS RelIO);
 void XGI_WaitEndRetrace(XGIIOADDRESS RelIO);
 
+#include <unistd.h>
 /* 2005/11/21 added by jjtseng */
 #define DelayS(sec) usleep((sec)*1000000)
 #define DelayMS(millisec) usleep((millisec)*1000)
@@ -871,6 +1015,9 @@ extern void XGI_SetRegANDOR(XGIIOADDRESS Port, USHORT Index, USHORT DataAND,
 extern void XGI_SetRegAND(XGIIOADDRESS Port, USHORT Index, USHORT DataAND);
 extern void XGI_SetRegOR(XGIIOADDRESS Port, USHORT Index, USHORT DataOR);
 
+#ifndef uint8_t
+#define uint8_t	CARD8
+#endif
 extern void XGI_WriteDAC(XGIIOADDRESS dac_data, unsigned shift,
     unsigned ordering, uint8_t red, uint8_t green, uint8_t blue);
 
